/** 
 *  Arduino UNO (remote receiver/ diaprojector controller) 
 *  Classic MIDI IN receiver via optocoupler - Rx arduino pin 0 to receive as standard midi in
 *  output to 1 midi in led and 6 control output leds
 *  use optocoupler triacs to trigger pulses on 6 outputs
 *  C P 2021
**/

const bool test = false; //true; //false;

// defines default midi in channel - default for MIDI library 
const byte midich = 1;

bool midibaudrate = true; //true;
// for testing over serial console
unsigned long baudrate = 115200;

#include <MIDI.h>

struct MySettings : public midi::DefaultSettings
{
   // do not conserve speed by not sending complete messages! 
   static const bool UseRunningStatus = false;
};

MIDI_CREATE_CUSTOM_INSTANCE(HardwareSerial, Serial, MIDI, MySettings);

// midi in led pin 
byte midiPin = A5;

// UNO receiver/ controller - pins 2, 3, 4, 5, 6, 7 defined as relay trigger outputs 
// for notes 1, 2, 3, 4, 5, 6
byte outPins[6] = {2, 3, 4, 5, 6, 7};
// control leds - yellow
byte ledPins[6] = {8, 9, 10, 11, 12, 13};

// global to hold data
byte data[6]={0,0,0,0,0,0};
// remember previous sensor value
byte previous[6]={0,0,0,0,0,0};

const unsigned long pulselength = 250; // 250 milliseconds
unsigned long pulsestartstamp[6]={millis(), millis(), millis(), millis(), millis(), millis()};
unsigned long midistartstamp = millis();

void handleNoteOn(byte inChannel, byte inNumber, byte inVelocity) {
  if(test) {
    Serial.print("NoteOn  ");
    Serial.print(inNumber);
    Serial.print("\tvelocity: ");
    Serial.println(inVelocity);
    
  } else {
    // Get the first data byte of the last received message - midi note 
    byte note = MIDI.getData1(); // get the note value 
    // Get the second data byte of the last received message - midi velocity
    // we only check 0 or 1 
    byte velocity = MIDI.getData2(); // get the note velocity
    // we have notes 1 to 6 with velocity 1
    if(note > 0 && note <= 6 && velocity > 0){
      // change the global
      data[note-1] = 1; //velocity; //1 
      //previous[note-1] = 1; // describes the note just sent
    } 
    
  }
}
void handleNoteOff(byte inChannel, byte inNumber, byte inVelocity) {
  if(test) {
    Serial.print("NoteOff ");
    Serial.print(inNumber);
    Serial.print("\tvelocity: ");
    Serial.println(inVelocity);
  } else {
    // Get the first data byte of the last received message - midi note 
    byte note = MIDI.getData1(); // get the note value 
    // Get the second data byte of the last received message - midi velocity
    // we only check 0 or 1 
    byte velocity = MIDI.getData2(); // get the note velocity
    // we get notes 1 to 6 with velocity 0
    if(note > 0 && note <= 6 && velocity==0){
      // change the global
      data[note-1] = 0;
      //previous[note-1] = 1; // describes the note just sent
    } 
  }
}

void setup() {
  // put your setup code here, to run once:

  // set outPins and ledPins as outputs
  for(byte i=0;i<sizeof(outPins);i++){
    pinMode(ledPins[i], OUTPUT);  
    pinMode(outPins[i], OUTPUT);  
    digitalWrite(ledPins[i], LOW);
    digitalWrite(outPins[i], LOW);
  }
     
  // midi in pin 
  pinMode(midiPin, OUTPUT);  
  digitalWrite(midiPin, LOW);

  // Initialize the serial port options - default is MIDI baudrate
  // here we have Serial on Rx!
  if(midibaudrate) {
    if(test){
      Serial.begin(baudrate); // serial console test via serial usb
      while (!Serial); // not needed here
      Serial.println("Serial connected");
    } else {
      // this we need
      Serial.begin(31250);    //  Set MIDI baud rate on rx tx pins
    }
  } else {
    Serial.begin(baudrate); // serial console test via serial usb
    Serial.println("Serial connected");
  }
   

  // midi in handlers
  MIDI.setHandleNoteOn(handleNoteOn);
  //MIDI.setHandleNoteOff(handleNoteOff);

  MIDI.begin();  // Launch MIDI and listen on default channel 1
  //MIDI.begin(midich); // listen to our channel only  
  //MIDI.begin(MIDI_CHANNEL_OMNI); // listen to all channels   

  // some delay
  delay(100);
  
  //do some intro dance
  for (byte i=0;i<sizeof(ledPins);i++){
    // blink
    digitalWrite(ledPins[i], HIGH);
    digitalWrite(midiPin, HIGH);
    delay(500);
    digitalWrite(ledPins[i], LOW);
    digitalWrite(midiPin, LOW);
    delay(500);
  }
  

}

void loop() {
  // we will not change this
  byte midi_channel = midich; // default is global

  // midiPin led blink is short
  if(millis()-midistartstamp > 100) { 
    digitalWrite(midiPin, LOW);
    midistartstamp = millis();
  }

  // stop trigger pulse outs and leds after 250ms
  for (byte ind=0;ind<6;ind++) {
    // ind is array index -> note 
    // stop the pulse and led after 250 milliseconds
    if(millis()-pulsestartstamp[ind] > pulselength) { 
       triggerPulse(ind, false); 
       // prepare for next pulse - declare previous 0
       //previous[ind]=0; // describes the pulse end reached
    }
  }

  MIDI.read();
  
  // now check data for any changed global in MIDI.read()
  for (byte ind=0;ind<6;ind++) {
    // ind is array index -> note 
    // data[ind] is velocity
    // trigger on pulse on relevant output pin - we need current index
    // will also blink corresponding led
    // check velocity data[ind]>0
    //if(data[ind]>0 && previous[ind]>0) {
    if(data[ind]>0) {
       triggerPulse(ind, true); 
       // remember start time 
       pulsestartstamp[ind]=millis();
       //previous[ind]=data[ind];
       data[ind]=0; // reset now that pulse was triggered
    }
  }
}

void triggerPulse(byte ind, bool on) {
  // ind corresponds to outPins index
  if(on){
    digitalWrite(outPins[ind], HIGH);
    digitalWrite(ledPins[ind], HIGH);
    digitalWrite(midiPin, HIGH);
  } else {
    digitalWrite(outPins[ind], LOW);
    digitalWrite(ledPins[ind], LOW);
  }
}
